# -*- encoding: utf-8 -*-
from __future__ import unicode_literals
from __future__ import absolute_import

import datetime

from django import template
from django.db.models import F
from django.utils.dateformat import format

from djapps.blogs.models import Blog, Post

register = template.Library()


@register.assignment_tag
def blog_years(blog, order='DESC'):
    if not blog:
        qsroot = Post.objects
    else:
        qsroot = blog.posts

    qs = qsroot.active().dates('date', 'year', order=order)
    return [dt.year for dt in qs]


@register.assignment_tag
def blog_months(blog, date=None, year=None, order='DESC'):
    if not blog:
        qsroot = Post.objects
    else:
        qsroot = blog.posts

    if date:
        year = date.year
    elif not year:
        return

    qs = qsroot.active().filter(
        date__year=year,
    ).dates('date', 'month', order=order)
    return [dt.month for dt in qs]


@register.assignment_tag
def blog_days(blog, date=None, year=None, month=None, order='DESC'):
    if not blog:
        qsroot = Post.objects
    else:
        qsroot = blog.posts

    if date:
        year = date.year
        month = date.month
    elif not year or not month:
        return

    qs = qsroot.active().filter(
        date__year=year,
        date__month=month,
    ).dates('date', 'day', order=order)
    return [dt.day for dt in qs]

@register.assignment_tag(takes_context=True)
def blogs_available(context):
    return Blog.objects.all()

@register.assignment_tag(takes_context=True)
def blogs_current_blog(context):
    try:
        return context['request']._blogs_current_blog
    except:
        pass

@register.assignment_tag(takes_context=True)
def blogs_current_date(context):
    try:
        return context['request']._blogs_current_date
    except:
        pass

@register.assignment_tag(takes_context=True)
def blogs_current_year(context):
    try:
        return context['request']._blogs_current_year
    except:
        pass

@register.assignment_tag(takes_context=True)
def blogs_current_month(context):
    try:
        return context['request']._blogs_current_month
    except:
        pass

@register.assignment_tag(takes_context=True)
def blogs_current_day(context):
    try:
        return context['request']._blogs_current_day
    except:
        pass

@register.filter()
def month_name(month_number):
    date = datetime.date(year=2000, month=int(month_number), day=1)
    return format(date, "F")



from django.db.models import signals

from feincms.management.checker import check_database_schema

from djapps.blogs.abstract_models import AbstractBlog, AbstractPost, PostManager


class Blog(AbstractBlog):
    pass


class Post(AbstractPost):
    pass


signals.post_syncdb.connect(
    check_database_schema(Post, __name__),
    weak=False
)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible

from feincms_template_content.models import TemplateContent

from djapps.blogs.models import Blog, Post
from djapps.blogs.views import PostView


@python_2_unicode_compatible
class RecentPostsListContent(TemplateContent):
    class Meta(object):
        abstract = True
        verbose_name = 'recent posts list'
        verbose_name_plural = 'recent posts lists'

    blog = models.ForeignKey(Blog, verbose_name=_('blog'),
                             blank=True, null=True)
    max_posts = models.PositiveIntegerField(_('posts'), default=5)
    offeset = models.PositiveIntegerField(_('offeset'), default=0)

    def __str__(self):
        return u"%d blog posts from %s" % (
            self.max_posts or 0,
            self.blog if self.blog_id else "all blogs",
        )

    def posts(self):
        if self.blog:
            posts = self.blog.posts.active()
        else:
            posts = Post.objects.active()
        return posts[self.offeset:self.offeset+self.max_posts]

class BlogArchiveContent(TemplateContent):
    class Meta(object):
        abstract = True
        verbose_name = 'blog archive'
        verbose_name_plural = 'blog archives'

    blog = models.ForeignKey(Blog, verbose_name=_('blog'),
                             blank=True, null=True)

    def __str__(self):
        return u"%s archive %s" % (
            self.blog if self.blog_id else None,
        )



import datetime

from django.views.generic.detail import SingleObjectMixin
from django.views.generic.list import ListView
from django.views.generic.base import View
from django.utils.translation import ugettext as _
from django.http import Http404, HttpResponseRedirect
from django.core.urlresolvers import reverse

from feincms.module.mixins import ContentView

from djapps.blogs.models import Blog, Post


class PostPermalinkView(SingleObjectMixin, View):
    model = Post

    def get_queryset(self):
        return self.model.objects.active()

    def get(self, request, *args, **kwargs):
        post = self.get_object()

        return HttpResponseRedirect(
            post.get_pretty_url()
        )


class PostView(ContentView):
    model = Post
    context_object_name = 'post'

    def get_queryset(self):
        return self.model.objects.active().filter(**self.kwargs)

    def get_context_data(self, *args, **kwargs):
        data = super(PostView, self).get_context_data(*args, **kwargs)
        self.request._blogs_current_blog = self.object.blog
        self.request._blogs_current_date = self.object.date
        self.request._blogs_current_year = int(self.kwargs.get('date__year', 0))
        self.request._blogs_current_month = int(self.kwargs.get('date__month', 0))
        self.request._blogs_current_day = int(self.kwargs.get('date__day', 0))
        return data

    def get_object(self):
        queryset = self.get_queryset()

        obj = queryset.first()
        if obj is None:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


class PostListView(ListView):
    model = Post
    paginate_by = 15

    def dispatch(self, *args, **kwargs):
        try:
            self.range_start = datetime.date(
                year=int(self.kwargs.get("date__year", 1)),
                month=int(self.kwargs.get("date__month", 1)),
                day=int(self.kwargs.get("date__day", 1)),
            )
        except ValueError:
            raise Http404(_("Invalid date"))

        return super(PostListView, self).dispatch(*args, **kwargs)

    def get_queryset(self):
        return self.model.objects.active().filter(
            **dict(
                (k, v) for k,v in self.kwargs.items()
            )
        )

    def get_context_data(self, *args, **kwargs):
        data = super(PostListView, self).get_context_data(*args, **kwargs)
        data["range_start"] = self.range_start
        data["year"] = self.kwargs.get('date__year', None)
        data["month"] = self.kwargs.get('date__month', None)
        data["day"] = self.kwargs.get('date__day', None)
        blogslug = self.kwargs.get('blog__slug', False)
        if blogslug:
            blog = Blog.objects.filter(slug=blogslug).first()
            if blog is None:
                raise Http404(_("Blog not found"))
            data["blog"] = blog
            self.request._blogs_current_blog = blog
        self.request._blogs_current_date = self.range_start
        self.request._blogs_current_year = int(self.kwargs.get('date__year', 0))
        self.request._blogs_current_month = int(self.kwargs.get('date__month', 0))
        self.request._blogs_current_day = int(self.kwargs.get('date__day', 0))
        return data



import operator
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible
from django.shortcuts import resolve_url

from feincms.models import create_base_model
from feincms.utils.managers import ActiveAwareContentManagerMixin
from feincms.module.mixins import ContentModelMixin


@python_2_unicode_compatible
class AbstractBlog(models.Model):
    class Meta(object):
        abstract = True
        ordering = 'name',

    name = models.CharField(_('name'), max_length=255, unique=True)
    slug = models.SlugField(_('slug'), unique=True)

    def __str__(self):
        return self.name


class PostManager(ActiveAwareContentManagerMixin, models.Manager):
    def after(self, post):
        filters = [Q(date__gt=post.date)]
        if post.time is not None:
            filters.append(Q(date=post.date, time__gt=post.time))
            filters.append(Q(date=post.date, time=post.time, id__gt=post.id))
        else:
            filters.append(Q(date=post.date, time__isnull=False))
            filters.append(Q(date=post.date, time__isnull=True, id__gt=post.id))

        return self.filter(reduce(operator.or_, filters))

    def before(self, post):
        filters = [Q(date__lt=post.date)]
        if post.time is not None:
            filters.append(Q(date=post.date, time__isnull=True))
            filters.append(Q(date=post.date, time__lt=post.time))
            filters.append(Q(date=post.date, time=post.time, id__lt=post.id))
        else:
            filters.append(Q(date=post.date, time__isnull=True, id__lt=post.id))

        return self.filter(reduce(operator.or_, filters))

PostManager.add_to_active_filters(Q(published=True))


@python_2_unicode_compatible
class AbstractPost(create_base_model(), ContentModelMixin):
    class Meta(object):
        abstract = True
        unique_together = (
            ('blog', 'slug', 'date'),
        )
        ordering = '-date', '-time', '-id'

    blog = models.ForeignKey(
        'Blog',
        related_name='posts',
        verbose_name=_('blog'),
        on_delete=models.PROTECT,
    )
    title = models.CharField(_('title'), max_length=255)
    slug = models.SlugField(_('slug'))
    published = models.BooleanField(_('published'), default=True)
    date = models.DateField(
        _('date'),
        default=lambda: timezone.localtime(timezone.now()).date(),
    )
    time = models.TimeField(
        _('time'),
        default=lambda: timezone.localtime(timezone.now()).time(),
        null=True, blank=True
    )

    objects = PostManager()

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return resolve_url("blogs:post_permalink", pk=self.pk)

    def get_pretty_url(self):
        return reverse(
            "blogs:post",
            args=[
                self.blog.slug,
                self.date.year,
                self.date.month,
                self.date.day,
                self.slug,
            ]
        )

    def next(self):
        return self.blog.posts.after(self).last()

    def previous(self):
        return self.blog.posts.before(self).first()



from django.contrib import admin
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.http import urlquote
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned

from feincms.admin import item_editor
from feincms import ensure_completely_loaded

from djapps.blogs.models import Blog, Post


class BlogAdmin(admin.ModelAdmin):
    list_display = [
        'name',
        'slug',
        'posts',
        'new_post',
    ]
    search_fields = [
        'name',
        'slug'
    ]
    prepopulated_fields = {'slug': ('name',)}

    def posts(self, obj):
        """Provides a link to the blog specific post admin changelist page"""
        return mark_safe(
            '<a href="%s?blog__id__exact=%d">%d posts...</a>' % (
                escape(reverse("admin:blogs_post_changelist")),
                obj.id,
                obj.posts.count(),
            )
        )

    def new_post(self, obj):
        """Provides a link to the blog specific post admin add page"""
        return mark_safe(
            '<a href="%s?_changelist_filters=%s">New post...</a>' % (
                escape(reverse("admin:blogs_post_add")),
                urlquote("blog__id__exact=%d" % obj.id),
            )
        )


class PostAdmin(item_editor.ItemEditor):
    date_hierarchy = 'date'
    list_display = [
        'title',
        'blog',
        'date',
        'time',
        'published',
    ]
    list_filter = [
        'blog',
        'published',
        'date',
    ]
    search_fields = [
        'title',
        'slug',
    ]
    prepopulated_fields = {'slug': ('title',)}
    radio_fields = {'template_key': admin.HORIZONTAL}

    fieldset_insertion_index = 2
    fieldsets = [
        (None, {
            'fields': [
                'blog',
                ('title', 'slug'),
                ('published'),
            ],
        }),
        ('Other options', {
            'classes': ['collapse'],
            'fields': ['template_key', ['date', 'time']],
        }),
        # <-- insertion point, extensions appear here, see insertion_index
        # above
        item_editor.FEINCMS_CONTENT_FIELDSET,
    ]

    def __init__(self, model, admin_site):
        ensure_completely_loaded()

        if len(model._feincms_templates) > 4 and \
                'template_key' in self.radio_fields:
            del(self.radio_fields['template_key'])

        super(PostAdmin, self).__init__(model, admin_site)

    def add_view(self, request, *args, **kwargs):
        """Add initial blog value if needed"""
        old_GET = request.GET
        GET = request.GET.copy()
        if 'blog' not in request.GET.keys():
            filters = GET.get('_changelist_filters', '')
            filters = filters.split('&')
            filters = [f.split('=', 1) for f in filters]
            filters = [f for f in filters if len(f) == 2]
            filters = dict(filters)
            blog_id = filters.get('blog__id__exact', None)
            if blog_id is None:
                try:
                    blog_id = Blog.objects.get().id
                except (ObjectDoesNotExist, MultipleObjectsReturned):
                    pass

            if blog_id is not None:
                GET['blog'] = blog_id

        try:
            request.GET = GET
            return super(PostAdmin, self).add_view(request, *args, **kwargs)
        finally:
            request.GET = old_GET


if getattr(settings, "FEINCMS_BLOGS_USE_BLOG_ADMIN", True):
    admin.site.register(Blog, BlogAdmin)

if getattr(settings, "FEINCMS_BLOGS_USE_POST_ADMIN", True):
    admin.site.register(Post, PostAdmin)



import datetime
from dateutil.relativedelta import relativedelta
from django.conf import settings

date = datetime.date(year=2012, month=3, day=3)
day = relativedelta(days=1)
month = relativedelta(months=1)
year = relativedelta(years=1)

settings.configure(
    DATABASES={'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }},
    INSTALLED_APPS=(
        'django.contrib.contenttypes',
        'feincms',
        'blogs',
    ),
    USE_TZ = True,
)

from django.test import TestCase
from django.core.management import call_command
from djapps.blogs.models import Blog, Post
from djapps.blogs.templatetags.blogs import blog_years, blog_months, blog_days
from djapps.blogs.views import PostView, PostListView
from django.test.client import RequestFactory
from feincms.content.richtext.models import RichTextContent

Post.register_templates(
    {'title': 'Page',
     'path': 'page/page.html',
     'regions': (
         ('main', 'Main content area'),
     )},
)
Post.create_content_type(RichTextContent)


call_command('syncdb', interactive=False)


class BlogTestCase(TestCase):
    def setUp(self):
        self.request_factory = RequestFactory()

    def test_active(self):
        blog = Blog.objects.create(name="test", slug="test")
        post = blog.posts.create()
        self.assertEqual(list(blog.posts.active()), [post])
        post.published = False
        post.save()
        self.assertEqual(list(blog.posts.active()), [])

    def test_before_after(self):

        blog = Blog.objects.create(name="test", slug="test")

        posts = [
            blog.posts.create(id=1,  title='1',  slug='1',
                              date=date-day, time=None),
            blog.posts.create(id=2,  title='2',  slug='2',
                              date=date-day, time=None),
            blog.posts.create(id=5,  title='5',  slug='5',
                              date=date-day, time=datetime.time(hour=11)),
            blog.posts.create(id=3,  title='3',  slug='3',
                              date=date-day, time=datetime.time(hour=12)),
            blog.posts.create(id=4,  title='4',  slug='4',
                              date=date-day, time=datetime.time(hour=12)),
            blog.posts.create(id=6,  title='6',  slug='6',
                              date=date-day, time=datetime.time(hour=13)),
            blog.posts.create(id=7,  title='7',  slug='7',
                              date=date, time=None),
            blog.posts.create(id=8,  title='8',  slug='8',
                              date=date, time=None),
            blog.posts.create(id=11, title='11', slug='11',
                              date=date, time=datetime.time(hour=11)),
            blog.posts.create(id=9,  title='9',  slug='9',
                              date=date, time=datetime.time(hour=12)),
            blog.posts.create(id=10, title='10', slug='10',
                              date=date, time=datetime.time(hour=12)),
            blog.posts.create(id=12, title='12', slug='12',
                              date=date, time=datetime.time(hour=13)),
        ]

        for i in range(len(posts)):
            post = posts[i]
            if i + 1 < len(posts):
                self.assertEqual(Post.objects.after(post).last(), posts[i+1])
            else:
                self.assertEqual(Post.objects.after(post).last(), None)

        posts.reverse()

        for i in range(len(posts)):
            post = posts[i]
            if i + 1 < len(posts):
                self.assertEqual(Post.objects.before(post).first(), posts[i+1])
            else:
                self.assertEqual(Post.objects.before(post).first(), None)

    def test_next_prev(self):
        date = datetime.date.today()
        blog1 = Blog.objects.create(name="test1", slug="test1")
        blog2 = Blog.objects.create(name="test2", slug="test2")
        posts = [
            blog1.posts.create(id=1,  title='1', slug='1', date=date),
            blog2.posts.create(id=2,  title='2', slug='2', date=date),
            blog1.posts.create(id=3,  title='3', slug='3', date=date),
        ]
        self.assertEqual(posts[0].next(), posts[2])
        self.assertEqual(posts[2].previous(), posts[0])
        self.assertEqual(posts[1].next(), None)
        self.assertEqual(posts[1].previous(), None)

    def test_blog_years(self):
        blog = Blog.objects.create(name="test", slug="test")
        blog.posts.create(slug="1", date=date)
        blog.posts.create(slug="2", date=date+day)
        blog.posts.create(slug="3", date=date+year)
        blog.posts.create(slug="4", date=date-year, published=False)
        blog2 = Blog.objects.create(name="test2", slug="test2")
        posts = [
            blog2.posts.create(slug="1", date=date),
            blog2.posts.create(slug="2", date=date-year-year),
        ]
        self.assertEqual(blog_years(blog),
                         [date.year + 1, date.year])
        self.assertEqual(blog_years(blog, order='ASC'),
                         [date.year, date.year + 1])
        self.assertEqual(blog_years(''), [date.year + 1,
                                          date.year,
                                          date.year - 2])

    def test_blog_months(self):
        blog = Blog.objects.create(name="test", slug="test")
        blog.posts.create(slug="1", date=date)
        blog.posts.create(slug="2", date=date+day)
        blog.posts.create(slug="3", date=date+month)
        blog.posts.create(slug="3", date=date+year)
        blog.posts.create(slug="3", date=date-year)
        blog.posts.create(slug="4", date=date-month, published=False)
        blog2 = Blog.objects.create(name="test2", slug="test2")
        posts = [
            blog2.posts.create(slug="1", date=date),
            blog2.posts.create(slug="2", date=date-month),
        ]
        self.assertEqual(blog_months(blog, date=date),
                         [date.month + 1, date.month])
        self.assertEqual(blog_months(blog, year=date.year),
                         [date.month + 1, date.month])
        self.assertEqual(blog_months(blog, date=date, order='ASC'),
                         [date.month, date.month + 1])
        self.assertEqual(blog_months(blog, year=date.year, order='ASC'),
                         [date.month, date.month + 1])
        self.assertEqual(blog_months('', date=date),
                         [date.month + 1,
                          date.month,
                          date.month - 1])
        self.assertEqual(blog_months(blog, date=''), None)
        self.assertEqual(blog_months('', year=date.year),
                         [date.month + 1,
                          date.month,
                          date.month - 1])
        self.assertEqual(blog_months(blog, year=''), None)

    def test_blog_days(self):
        blog = Blog.objects.create(name="test", slug="test")
        blog.posts.create(slug="1", date=date)
        blog.posts.create(slug="2", date=date)
        blog.posts.create(slug="2", date=date+day)
        blog.posts.create(slug="3", date=date+month)
        blog.posts.create(slug="3", date=date-month)
        blog.posts.create(slug="3", date=date+year)
        blog.posts.create(slug="3", date=date-year)
        blog.posts.create(slug="4", date=date-day, published=False)
        blog2 = Blog.objects.create(name="test2", slug="test2")
        posts = [
            blog2.posts.create(slug="1", date=date),
            blog2.posts.create(slug="2", date=date-month),
            blog2.posts.create(slug="3", date=date - day - day),
        ]
        self.assertEqual(blog_days(blog, year=date.year, month=date.month),
                         [date.day + 1, date.day])
        self.assertEqual(blog_days(blog, date=date),
                         [date.day + 1, date.day])
        self.assertEqual(blog_days(blog, year=date.year,
                                   month=date.month, order='ASC'),
                         [date.day, date.day + 1])
        self.assertEqual(blog_days(blog, date=date, order='ASC'),
                         [date.day, date.day + 1])
        self.assertEqual(blog_days('', date=date),
                         [date.day + 1,
                          date.day,
                          date.day - 2])
        self.assertEqual(blog_days(blog, date=''), None)
        self.assertEqual(blog_days('', year=date.year, month=date.month),
                         [date.day + 1,
                          date.day,
                          date.day - 2])
        self.assertEqual(blog_days(blog, year='', month=date.month), None)
        self.assertEqual(blog_days(blog, year=date.year, month=''), None)
        self.assertEqual(blog_days(blog, year='', month=''), None)

    def test_view_post_variables(self):
        blog = Blog.objects.create(name="test", slug="test")
        post = blog.posts.create(slug="test", date=date)
        request = self.request_factory.get('/')
        PostView().dispatch(
            request,
            blog__slug='test',
            date__year=date.year,
            date__month=date.month,
            date__day=date.day,
            slug='test',
        ).context_data
        self.assertEqual(request._blogs_current_blog, blog)
        self.assertEqual(request._blogs_current_date, post.date)
        self.assertEqual(request._blogs_current_year, post.date.year)
        self.assertEqual(request._blogs_current_month, post.date.month)
        self.assertEqual(request._blogs_current_day, post.date.day)

    def test_view_list_variables(self):
        blog = Blog.objects.create(name="test", slug="test")

        request = self.request_factory.get('/')
        PostListView.as_view()(
            request=request,
            blog__slug='test',
        ).context_data
        self.assertEqual(request._blogs_current_blog, blog)
        self.assertEqual(request._blogs_current_date,
                         datetime.date(year=1, month=1, day=1))
        self.assertEqual(request._blogs_current_year, 0)
        self.assertEqual(request._blogs_current_month, 0)
        self.assertEqual(request._blogs_current_day, 0)

        request = self.request_factory.get('/')
        PostListView.as_view()(
            request=request,
            blog__slug='test',
            date__year=date.year,
        ).context_data
        self.assertEqual(request._blogs_current_blog, blog)
        self.assertEqual(request._blogs_current_date,
                         datetime.date(year=date.year, month=1, day=1))
        self.assertEqual(request._blogs_current_year, date.year)
        self.assertEqual(request._blogs_current_month, 0)
        self.assertEqual(request._blogs_current_day, 0)

        request = self.request_factory.get('/')
        PostListView.as_view()(
            request=request,
            blog__slug='test',
            date__year=date.year,
            date__month=date.month,
        )
        self.assertEqual(
            request._blogs_current_date,
            datetime.date(year=date.year, month=date.month, day=1)
        )
        self.assertEqual(request._blogs_current_year, date.year)
        self.assertEqual(request._blogs_current_month, date.month)
        self.assertEqual(request._blogs_current_day, 0)

        request = self.request_factory.get('/')
        PostListView.as_view()(
            request=request,
            blog__slug='test',
            date__year=date.year,
            date__month=date.month,
            date__day=date.day,
        )
        self.assertEqual(
            request._blogs_current_date,
            datetime.date(year=date.year, month=date.month, day=date.day)
        )
        self.assertEqual(request._blogs_current_year, date.year)
        self.assertEqual(request._blogs_current_month, date.month)
        self.assertEqual(request._blogs_current_day, date.day)



from setuptools import setup
import blogs

setup(
    name='FeinCMS-Blogs',
    version=blogs.__version__,
    packages=['blogs', 'blogs.templatetags'],
    license='BSD',
    description='Multiple blog/article app for Django/FeinCMS.',
    long_description=open('README.rst').read(),
    author='Richard Ward',
    author_email='richard.ward@fah-designs.co.uk',
    url='https://github.com/fah-designs/feincms-blogs',
    install_requires=['django', 'feincms', 'feincms-template-content', 'djapps'],
    test_suite='tests',
    tests_require=['python-dateutil'],
)



from django.conf.urls import patterns, url

from djapps.blogs.views import PostListView
from djapps.blogs.views import PostView
from djapps.blogs.views import PostPermalinkView

urlpatterns = patterns(
    '',
    url(r'^permalink/(?P<pk>[1-9][0-9]*)/$',
        PostPermalinkView.as_view(),
        name='post_permalink'),
    url(r'^all/all/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^all'
        r'/(?P<date__year>[1-9][0-9]{3})/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^all'
        r'/(?P<date__year>[1-9][0-9]{3})'
        r'/(?P<date__month>[1-9][0-9]?)/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^all'
        r'/(?P<date__year>[1-9][0-9]{3})'
        r'/(?P<date__month>[1-9][0-9]?)'
        r'/(?P<date__day>[1-9][0-9]?)/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^(?P<blog__slug>[^/]+)/all/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^(?P<blog__slug>[^/]+)'
        r'/(?P<date__year>[1-9][0-9]{3})/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^(?P<blog__slug>[^/]+)'
        r'/(?P<date__year>[1-9][0-9]{3})'
        r'/(?P<date__month>[1-9][0-9]?)/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^(?P<blog__slug>[^/]+)'
        r'/(?P<date__year>[1-9][0-9]{3})'
        r'/(?P<date__month>[1-9][0-9]?)'
        r'/(?P<date__day>[1-9][0-9]?)/$',
        PostListView.as_view(),
        name='post_list'),
    url(r'^(?P<blog__slug>[^/]+)'
        r'/(?P<date__year>[1-9][0-9]{3})'
        r'/(?P<date__month>[1-9][0-9]?)'
        r'/(?P<date__day>[1-9][0-9]?)'
        r'/(?P<slug>[^/]+)/$',
        PostView.as_view(),
        name='post'),
)
